@@ -53,14 +53,11 @@ struct l2tp_parm {
 	inet_prefix peer_ip;
 
 	uint16_t pw_type;
-	uint16_t mtu;
 	unsigned int udp6_csum_tx:1;
 	unsigned int udp6_csum_rx:1;
 	unsigned int udp_csum:1;
 	unsigned int recv_seq:1;
 	unsigned int send_seq:1;
-	unsigned int lns_mode:1;
-	unsigned int data_seq:2;
 	unsigned int tunnel:1;
 	unsigned int session:1;
 	int reorder_timeout;
@@ -159,16 +156,10 @@ static int create_session(struct l2tp_parm *p)
 	addattr8(&req.n, 1024, L2TP_ATTR_L2SPEC_TYPE, p->l2spec_type);
 	addattr8(&req.n, 1024, L2TP_ATTR_L2SPEC_LEN, p->l2spec_len);
 
-	if (p->mtu)
-		addattr16(&req.n, 1024, L2TP_ATTR_MTU, p->mtu);
 	if (p->recv_seq)
 		addattr8(&req.n, 1024, L2TP_ATTR_RECV_SEQ, 1);
 	if (p->send_seq)
 		addattr8(&req.n, 1024, L2TP_ATTR_SEND_SEQ, 1);
-	if (p->lns_mode)
-		addattr(&req.n, 1024, L2TP_ATTR_LNS_MODE);
-	if (p->data_seq)
-		addattr8(&req.n, 1024, L2TP_ATTR_DATA_SEQ, p->data_seq);
 	if (p->reorder_timeout)
 		addattr64(&req.n, 1024, L2TP_ATTR_RECV_TIMEOUT,
 					  p->reorder_timeout);
@@ -228,7 +219,7 @@ static void print_tunnel(const struct l2tp_data *data)
 	print_string(PRINT_ANY, "encap", " encap %s",
 		     p->encap == L2TP_ENCAPTYPE_UDP ? "UDP" :
 		     p->encap == L2TP_ENCAPTYPE_IP ? "IP" : "??");
-	print_string(PRINT_FP, NULL, "%s", _SL_);
+	print_nl();
 
 	print_string(PRINT_ANY, "local", "  From %s ",
 		     inet_ntop(p->local_ip.family, p->local_ip.data,
@@ -236,11 +227,11 @@ static void print_tunnel(const struct l2tp_data *data)
 	print_string(PRINT_ANY, "peer", "to %s",
 		     inet_ntop(p->peer_ip.family, p->peer_ip.data,
 			       buf, sizeof(buf)));
-	print_string(PRINT_FP, NULL, "%s", _SL_);
+	print_nl();
 
 	print_uint(PRINT_ANY, "peer_tunnel", "  Peer tunnel %u",
 		   p->peer_tunnel_id);
-	print_string(PRINT_FP, NULL, "%s", _SL_);
+	print_nl();
 
 	if (p->encap == L2TP_ENCAPTYPE_UDP) {
 		print_string(PRINT_FP, NULL,
@@ -250,7 +241,7 @@ static void print_tunnel(const struct l2tp_data *data)
 			   p->local_udp_port);
 		print_uint(PRINT_ANY, "peer_port", "/%hu",
 			   p->peer_udp_port);
-		print_string(PRINT_FP, NULL, "%s", _SL_);
+		print_nl();
 
 		switch (p->local_ip.family) {
 		case AF_INET:
@@ -292,18 +283,18 @@ static void print_session(struct l2tp_data *data)
 
 	print_uint(PRINT_ANY, "session_id", "Session %u", p->session_id);
 	print_uint(PRINT_ANY, "tunnel_id",  " in tunnel %u", p->tunnel_id);
-	print_string(PRINT_FP, NULL, "%s", _SL_);
+	print_nl();
 
 	print_uint(PRINT_ANY, "peer_session_id",
 		     "  Peer session %u,", p->peer_session_id);
 	print_uint(PRINT_ANY, "peer_tunnel_id",
 		     " tunnel %u",  p->peer_tunnel_id);
-	print_string(PRINT_FP, NULL, "%s", _SL_);
+	print_nl();
 
 	if (p->ifname != NULL) {
 		print_color_string(PRINT_ANY, COLOR_IFNAME,
 				   "interface", "  interface name: %s" , p->ifname);
-		print_string(PRINT_FP, NULL, "%s", _SL_);
+		print_nl();
 	}
 
 	/* Show offsets only for plain console output (for legacy scripts) */
@@ -359,8 +350,6 @@ static int get_response(struct nlmsghdr *n, void *arg)
 		p->pw_type = rta_getattr_u16(attrs[L2TP_ATTR_PW_TYPE]);
 	if (attrs[L2TP_ATTR_ENCAP_TYPE])
 		p->encap = rta_getattr_u16(attrs[L2TP_ATTR_ENCAP_TYPE]);
-	if (attrs[L2TP_ATTR_DATA_SEQ])
-		p->data_seq = rta_getattr_u16(attrs[L2TP_ATTR_DATA_SEQ]);
 	if (attrs[L2TP_ATTR_CONN_ID])
 		p->tunnel_id = rta_getattr_u32(attrs[L2TP_ATTR_CONN_ID]);
 	if (attrs[L2TP_ATTR_PEER_CONN_ID])
@@ -418,8 +407,6 @@ static int get_response(struct nlmsghdr *n, void *arg)
 		p->local_udp_port = rta_getattr_u16(attrs[L2TP_ATTR_UDP_SPORT]);
 	if (attrs[L2TP_ATTR_UDP_DPORT])
 		p->peer_udp_port = rta_getattr_u16(attrs[L2TP_ATTR_UDP_DPORT]);
-	if (attrs[L2TP_ATTR_MTU])
-		p->mtu = rta_getattr_u16(attrs[L2TP_ATTR_MTU]);
 	if (attrs[L2TP_ATTR_IFNAME])
 		p->ifname = rta_getattr_str(attrs[L2TP_ATTR_IFNAME]);
 
